@@ -5,14 +5,15 @@
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-
 import re
-import pickle as pickle
 
+from six import iteritems, itervalues, text_type, string_types
+from six.moves import cPickle as pickle
 from docutils.nodes import raw, comment, title, Text, NodeVisitor, SkipNode
+from os import path
 
 from sphinx.util import jsdump, rpartition
 
@@ -40,7 +41,9 @@ class SearchLanguage(object):
        type, before searching index. Default implementation does nothing.
     """
     lang = None
+    language_name = None
     stopwords = set()
+    js_stemmer_rawcode = None
     js_stemmer_code = """
 /**
  * Dummy stemmer for languages without stemming rules.
@@ -89,16 +92,49 @@ var Stemmer = function() {
         Return true if the target word should be registered in the search index.
         This method is called after stemming.
         """
-        return not (((len(word) < 3) and (12353 < ord(word[0]) < 12436)) or
-            (ord(word[0]) < 256 and (len(word) < 3 or word in self.stopwords or
-                                     word.isdigit())))
+        return (
+            len(word) == 0 or not (
+                ((len(word) < 3) and (12353 < ord(word[0]) < 12436)) or
+                (ord(word[0]) < 256 and (
+                    len(word) < 3 or word in self.stopwords or word.isdigit()
+                ))))
 
 
-from sphinx.search import en, ja
+# SearchEnglish imported after SearchLanguage is defined due to circular import
+from sphinx.search.en import SearchEnglish
 
+
+def parse_stop_word(source):
+    """
+    parse snowball style word list like this:
+
+    * http://snowball.tartarus.org/algorithms/finnish/stop.txt
+    """
+    result = set()
+    for line in source.splitlines():
+        line = line.split('|')[0]  # remove comment
+        result.update(line.split())
+    return result
+
+
+# maps language name to module.class or directly a class
 languages = {
-    'en': en.SearchEnglish,
-    'ja': ja.SearchJapanese,
+    'da': 'sphinx.search.da.SearchDanish',
+    'de': 'sphinx.search.de.SearchGerman',
+    'en': SearchEnglish,
+    'es': 'sphinx.search.es.SearchSpanish',
+    'fi': 'sphinx.search.fi.SearchFinnish',
+    'fr': 'sphinx.search.fr.SearchFrench',
+    'hu': 'sphinx.search.hu.SearchHungarian',
+    'it': 'sphinx.search.it.SearchItalian',
+    'ja': 'sphinx.search.ja.SearchJapanese',
+    'nl': 'sphinx.search.nl.SearchDutch',
+    'no': 'sphinx.search.no.SearchNorwegian',
+    'pt': 'sphinx.search.pt.SearchPortuguese',
+    'ro': 'sphinx.search.ro.SearchRomanian',
+    'ru': 'sphinx.search.ru.SearchRussian',
+    'sv': 'sphinx.search.sv.SearchSwedish',
+    'tr': 'sphinx.search.tr.SearchTurkish',
 }
 
 
@@ -143,9 +179,10 @@ class WordCollector(NodeVisitor):
         self.lang = lang
 
     def dispatch_visit(self, node):
-        if node.__class__ is comment:
+        nodetype = type(node)
+        if issubclass(nodetype, comment):
             raise SkipNode
-        if node.__class__ is raw:
+        if issubclass(nodetype, raw):
             # Some people might put content in raw HTML that should be searched,
             # so we just amateurishly strip HTML tags and index the remaining
             # content
@@ -154,9 +191,9 @@ class WordCollector(NodeVisitor):
             nodetext = re.sub(r'<[^<]+?>', '', nodetext)
             self.found_words.extend(self.lang.split(nodetext))
             raise SkipNode
-        if node.__class__ is Text:
+        if issubclass(nodetype, Text):
             self.found_words.extend(self.lang.split(node.astext()))
-        elif node.__class__ is title:
+        elif issubclass(nodetype, title):
             self.found_title_words.extend(self.lang.split(node.astext()))
 
 
@@ -185,17 +222,27 @@ class IndexBuilder(object):
         # objtype index -> (domain, type, objname (localized))
         self._objnames = {}
         # add language-specific SearchLanguage instance
-        self.lang = languages[lang](options)
+        lang_class = languages.get(lang)
+        if lang_class is None:
+            self.lang = SearchEnglish(options)
+        elif isinstance(lang_class, str):
+            module, classname = lang_class.rsplit('.', 1)
+            lang_class = getattr(__import__(module, None, None, [classname]),
+                                 classname)
+            self.lang = lang_class(options)
+        else:
+            # it's directly a class (e.g. added by app.add_search_language)
+            self.lang = lang_class(options)
 
         if scoring:
             with open(scoring, 'rb') as fp:
                 self.js_scorer_code = fp.read().decode('utf-8')
         else:
-            self.js_scorer_code = ''
+            self.js_scorer_code = u''
 
     def load(self, stream, format):
         """Reconstruct from frozen data."""
-        if isinstance(format, str):
+        if isinstance(format, string_types):
             format = self.formats[format]
         frozen = format.load(stream)
         # if an old index is present, we treat it as not existing.
@@ -203,11 +250,11 @@ class IndexBuilder(object):
            frozen.get('envversion') != self.env.version:
             raise ValueError('old format')
         index2fn = frozen['filenames']
-        self._titles = dict(list(zip(index2fn, frozen['titles'])))
+        self._titles = dict(zip(index2fn, frozen['titles']))
 
         def load_terms(mapping):
             rv = {}
-            for k, v in mapping.items():
+            for k, v in iteritems(mapping):
                 if isinstance(v, int):
                     rv[k] = set([index2fn[v]])
                 else:
@@ -220,7 +267,7 @@ class IndexBuilder(object):
 
     def dump(self, stream, format):
         """Dump the frozen index to a stream."""
-        if isinstance(format, str):
+        if isinstance(format, string_types):
             format = self.formats[format]
         format.dump(self.freeze(), stream)
 
@@ -228,7 +275,7 @@ class IndexBuilder(object):
         rv = {}
         otypes = self._objtypes
         onames = self._objnames
-        for domainname, domain in self.env.domains.items():
+        for domainname, domain in iteritems(self.env.domains):
             for fullname, dispname, type, docname, anchor, prio in \
                     domain.get_objects():
                 # XXX use dispname?
@@ -247,7 +294,7 @@ class IndexBuilder(object):
                     if otype:
                         # use unicode() to fire translation proxies
                         onames[typeindex] = (domainname, type,
-                            str(domain.get_type_name(otype)))
+                                             text_type(domain.get_type_name(otype)))
                     else:
                         onames[typeindex] = (domainname, type, type)
                 if anchor == fullname:
@@ -262,30 +309,32 @@ class IndexBuilder(object):
     def get_terms(self, fn2index):
         rvs = {}, {}
         for rv, mapping in zip(rvs, (self._mapping, self._title_mapping)):
-            for k, v in mapping.items():
+            for k, v in iteritems(mapping):
                 if len(v) == 1:
                     fn, = v
                     if fn in fn2index:
                         rv[k] = fn2index[fn]
                 else:
-                    rv[k] = [fn2index[fn] for fn in v if fn in fn2index]
+                    rv[k] = sorted([fn2index[fn] for fn in v if fn in fn2index])
         return rvs
 
     def freeze(self):
         """Create a usable data structure for serializing."""
-        filenames = list(self._titles.keys())
-        titles = list(self._titles.values())
+        filenames, titles = zip(*sorted(self._titles.items()))
         fn2index = dict((f, i) for (i, f) in enumerate(filenames))
         terms, title_terms = self.get_terms(fn2index)
 
         objects = self.get_objects(fn2index)  # populates _objtypes
         objtypes = dict((v, k[0] + ':' + k[1])
-                        for (k, v) in self._objtypes.items())
+                        for (k, v) in iteritems(self._objtypes))
         objnames = self._objnames
         return dict(filenames=filenames, titles=titles, terms=terms,
                     objects=objects, objtypes=objtypes, objnames=objnames,
                     titleterms=title_terms, envversion=self.env.version)
 
+    def label(self):
+        return "%s (code: %s)" % (self.lang.language_name, self.lang.lang)
+
     def prune(self, filenames):
         """Remove data for all filenames not in the list."""
         new_titles = {}
@@ -293,9 +342,9 @@ class IndexBuilder(object):
             if filename in self._titles:
                 new_titles[filename] = self._titles[filename]
         self._titles = new_titles
-        for wordnames in self._mapping.values():
+        for wordnames in itervalues(self._mapping):
             wordnames.intersection_update(filenames)
-        for wordnames in self._title_mapping.values():
+        for wordnames in itervalues(self._title_mapping):
             wordnames.intersection_update(filenames)
 
     def feed(self, filename, title, doctree):
@@ -312,7 +361,7 @@ class IndexBuilder(object):
             except KeyError:
                 self._stem_cache[word] = self.lang.stem(word)
                 return self._stem_cache[word]
-        _filter =  self.lang.word_filter
+        _filter = self.lang.word_filter
 
         for word in visitor.found_title_words:
             word = stem(word)
@@ -327,7 +376,14 @@ class IndexBuilder(object):
     def context_for_searchtool(self):
         return dict(
             search_language_stemming_code = self.lang.js_stemmer_code,
-            search_language_stop_words =
-                jsdump.dumps(sorted(self.lang.stopwords)),
+            search_language_stop_words = jsdump.dumps(sorted(self.lang.stopwords)),
             search_scorer_tool = self.js_scorer_code,
         )
+
+    def get_js_stemmer_rawcode(self):
+        if self.lang.js_stemmer_rawcode:
+            return path.join(
+                path.dirname(path.abspath(__file__)),
+                'non-minified-js',
+                self.lang.js_stemmer_rawcode
+            )
